angular.module('beamng.apps')

.directive('velocityForcedInduction', ['bngApi', 'StreamsManager', "UiUnits", function(bngApi, StreamsManager, UiUnits) {
  // noinspection CssUnknownTarget
  return {
    template:
        '<div>' +
          '<link rel="stylesheet" href="../ui/modules/apps/VelocityForcedInduction/app.css">' +
          '<link rel="stylesheet" href="../ui/tachometers/base.css">' +
          // Pre-load font
          '<canvas class="canvas" id="VFI_bgCanvas" width="150" height="150"></canvas>' +
          '<canvas class="canvas" id="VFI_canvas"   width="150" height="150"></canvas>' +
        '</div>',
    replace: true,
    restrict: 'EA',
    link: function ($scope, $element) {
      const ID = "VFI";
      const REQ_STREAMS = ["forcedInductionInfo"];
      const REQ_SERVICES = ["service", "velocityService"];

      StreamsManager.add(REQ_STREAMS);


      // Below has to be in all apps, and cannot be added to the BaseApp class since the file that
      // is in hasn't been loaded until this code has run

      let scriptsLoaded = 0;
      // Add in the script files for the base service
      for (let service of REQ_SERVICES) {
        let script = document.createElement("script");
        script.src = "../ui/tachometers/" + service + ".js";
        // Add in this directives ID to avoid conflicts between all of these modded apps
        script.id = service + ID;
        $element.append(script);

        document.getElementById(script.id).onload = function() {
          scriptsLoaded++;
          // Wait for all scripts to load
          if (scriptsLoaded === REQ_SERVICES.length)
            onload();
        };
      }


      let app = null;
      let TachoBase = null;

      //////////////////////////////
      // Rendering/Main Functions //
      //////////////////////////////

      /**
       * Returns a list of all dynamic variables that will/may change between frames for the vehicle
       * @param {Object[]} forcedInductionInfo - the forced induction information stream
       */
      function getDynamicVars(forcedInductionInfo) {
        return {
          boost: forcedInductionInfo.boost
        };
      }


      $scope.$on("streamsUpdate", function (event, streams) {
        let forcedInductionInfo = streams.forcedInductionInfo;

        // Wait until ready
        if (!app || app.state === app.states.preInit || !forcedInductionInfo)
          return;

        let baseVars = getDynamicVars(forcedInductionInfo);

        // Perform first time setup if necessary
        if (app.state === app.states.init) {
          app.initFields(baseVars.boost, forcedInductionInfo.maxBoost);
          app.load(bngApi);
          app.drawBackgroundCanvas();
          app.state = app.states.loaded;
        }

        app.setCurrBoost(baseVars.boost);
        app.draw();
      });

      $scope.$on("$destroy", () => StreamsManager.remove(REQ_STREAMS));
      $scope.$on("VehicleFocusChanged", () => TachoBase.BoostApp.reload(app, true));
      $scope.$on("VehicleChange", () => TachoBase.BoostApp.reload(app, false));

      /**
       * Loads the external services and initialises some base constants.
       */
      function onload() {
        // Request and setup the services
        let injector = angular.injector(["beamng.apps"]);
        let ModVelocityBase = injector.get("ModVelocityBase");
        TachoBase = ModVelocityBase.TachoBase;

        app = new ModVelocityBase.VelocityBoostApp(ID, document, bngApi, UiUnits);
      }
    }
  };
}]);